package com.muyu.analysis.parsing.mqtt.service;



import com.alibaba.fastjson2.JSONObject;
import com.muyu.analysis.parsing.mqtt.ParsingMQTT;
import com.muyu.common.core.constant.KafkaConstants;
import com.muyu.enterprise.domain.MqttProperties;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PreDestroy;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


/**
 * @Author：李庆帅
 * @Package：com.muyu.analysis.parsing.mqtt.service
 * @Project：cloud-server
 * @name：MqttClientService
 * @Date：2024/10/10 20:24
 */

@Service
@Slf4j
public class MqttClientService {

    @Autowired
    private ParsingMQTT messageProcessor;

    @Resource
    private KafkaProducer<String, String> kafkaProducer;

    private final ExecutorService executorService = Executors.newCachedThreadPool();
    private MqttClient sampleClient;


    public void connectAndSubscribeAsync(MqttProperties mqttProperties) {
        executorService.submit(() -> {
            try {
                connectAndSubscribe(mqttProperties);
            } catch (MqttException | IOException e) {
                log.error("MQTT连接或订阅失败", e);
            }
        });
    }

    private void connectAndSubscribe(MqttProperties mqttProperties) throws MqttException, IOException {
        if (sampleClient != null && sampleClient.isConnected()) {
            log.info("MQTT客户端已经连接，跳过重新连接。");
            return;
        }

        sampleClient = new MqttClient(mqttProperties.getBroker(), mqttProperties.getClientId());
        MqttConnectOptions connOpts = new MqttConnectOptions();
        connOpts.setUserName(mqttProperties.getUserName());
        connOpts.setPassword(mqttProperties.getPassword().toCharArray());
        connOpts.setCleanSession(true);

        sampleClient.connect(connOpts);
        sampleClient.subscribe(mqttProperties.getTopic(), 0);

        sampleClient.setCallback(new MqttCallback() {
            @Override
            public void connectionLost(Throwable throwable) {
                log.error("连接丢失：{}", throwable.getMessage());
            }

            @Override
            public void messageArrived(String s, MqttMessage mqttMessage) {
                executorService.submit(() -> messageProcessor.mqttClient(mqttMessage,new String(mqttMessage.getPayload())));
                System.out.println(new String(mqttMessage.getPayload()));
                JSONObject entries = messageProcessor.mqttClient(mqttMessage, new String(mqttMessage.getPayload()));

                ProducerRecord<String, String> producerRecord = new ProducerRecord<>(KafkaConstants.MESSAGE_PARSING,
                        entries.toString()      );
                kafkaProducer.send(producerRecord);
                log.info("解析之后的数据："+entries);

            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
                log.info("消息发送完成：{}", iMqttDeliveryToken);
            }
        });
    }

    @PreDestroy
    public void shutdown() {
        executorService.shutdown();
        if (sampleClient != null && sampleClient.isConnected()) {
            try {
                sampleClient.disconnect();
            } catch (MqttException e) {
                log.error("MQTT客户端断开连接失败", e);
            }
        }
    }


}
