package com.muyu.vehiclegateway.service.impl;

import com.muyu.common.core.domain.Result;
import com.muyu.common.redis.service.RedisService;
import com.muyu.vehiclegateway.config.ClientConfig;
import com.muyu.vehiclegateway.domain.MqttProperties;
import com.muyu.vehiclegateway.domain.MqttServerModel;
import com.muyu.vehiclegateway.domain.VinIp;
import com.muyu.vehiclegateway.domain.req.VehicleConnectionReq;
import com.muyu.vehiclegateway.instance.GenerateInstance;
import com.muyu.vehiclegateway.mapper.ConnectMapper;
import com.muyu.vehiclegateway.service.ConnectService;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.muyu.vehiclegateway.config.RabbitmqConfig.LINE_EXCHANGE;
import static com.muyu.vehiclegateway.config.RabbitmqConfig.SEND_ONLINE_ROUTING_KEY;


/**
 * @ClassName ConnectServiceImpl
 * @Description 描述
 * @Author YiBo.Liu
 * @Date 2024/10/2 16:25
 */
@Service
@Log4j2
public class ConnectServiceImpl implements ConnectService {

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private ConnectMapper connectMapper;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private ClientConfig clientConfig;

    @Autowired
    private RedisService redisService;

    /**
     * 创建实例
     * @throws Exception
     */
    @Override
    public void createConnect() throws Exception {
        GenerateInstance generateInstance = new GenerateInstance();
        generateInstance.generateInstance();
    }

    /**
     * 获取车辆信息
     * @param vehicleConnectionReq
     * @return
     */
    @Override
    public Result<MqttServerModel> receiveMsg(VehicleConnectionReq vehicleConnectionReq) throws Exception {

        log.info("车辆连接请求:{}", vehicleConnectionReq);

        //生成密码
        vehicleConnectionReq.setPassword(vehicleConnectionReq.getVehicleVin() + vehicleConnectionReq.getTimestamp()
                + vehicleConnectionReq.getNonce());

        //查询有没有这辆车的vin码
        List<String> selectVehicle =  connectMapper.selectVehicleVin(vehicleConnectionReq.getVehicleVin());

        if(selectVehicle.isEmpty()){
            connectMapper.addVehicle(vehicleConnectionReq);
            log.info("车辆预上线成功");
        }else {
            log.info("车辆无法重复预上线");
        }

        //先判断vin码
//        String vin = redisTemplate.opsForValue().get(vehicleConnectionReq.getVehicleVin());
        if(redisTemplate.hasKey(vehicleConnectionReq.getVehicleVin())){
            log.info("车辆绑定ip失败，已经存在");
            throw new RuntimeException("车辆已经绑定过了");
        }

        //判断redis有没有count键
        if(redisTemplate.hasKey("count")){
            //取出count
            Integer count = Integer.valueOf(redisTemplate.opsForValue().get("count"));
            if(count == 1){
                redisTemplate.opsForValue().set("count",String.valueOf(0));
            }else {
                redisTemplate.opsForValue().set("count",String.valueOf(count+1));
            }
            //根据游标count获取服务IP
            Object ip = redisService.redisTemplate.opsForList().index("ipList", count);
            //关联车辆和服务
            this.insertVinIp(new VinIp(vehicleConnectionReq.getVehicleVin(),ip.toString()));
            //响应信息
            log.info("车辆:"+vehicleConnectionReq.getVehicleVin()+"成功绑定到:"+ip);

            //发送mq到协议解析
            MqttProperties mqttProperties = new MqttProperties();
            mqttProperties.setBroker("tcp://" + ip + ":1883");
            mqttProperties.setTopic("vehicle");
            mqttProperties.setUsername(vehicleConnectionReq.getUsername());
            mqttProperties.setPassword(vehicleConnectionReq.getPassword());
            mqttProperties.setClientId(ip.toString());
            mqttProperties.setQos(0);

            rabbitTemplate.convertAndSend(LINE_EXCHANGE,SEND_ONLINE_ROUTING_KEY,mqttProperties);

            return Result.success(new MqttServerModel("tcp://"+ip+":1883","vehicle"));
        }else {
            redisTemplate.opsForValue().set("count",String.valueOf(0));
            //根据游标count获取服务器Ip
            Object ip = redisService.redisTemplate.opsForList().index("ipList", 0);
            log.info("ip为:{}",ip);
            //关联车辆和服务
            this.insertVinIp(new VinIp(vehicleConnectionReq.getVehicleVin(),ip.toString()));
            //响应信息
            log.info("车辆:{}",vehicleConnectionReq.getVehicleVin(),"成功绑定到:{}",ip);
            return Result.success(new MqttServerModel("tcp://"+ip+":1883","vehicle"));
        }
    }

    /**
     * 添加车辆绑定ip 存入redis
     */
    private void insertVinIp(VinIp vinIp){
        if (vinIp == null || vinIp.getVin() == null || vinIp.getVin().isEmpty() || vinIp.getIp() == null || vinIp.getIp().isEmpty()) {
            throw new IllegalArgumentException("vin 或 ip 不能为空或无效");
        }

        redisTemplate.opsForValue().set(vinIp.getVin(),vinIp.getIp());
    }

    /**
     * 增加服务器
     */
    private String addNewServer() throws Exception {
        // 这里调用你已经写好的方法来新增服务器
        // 假设新增服务器的方法是addServer，返回新服务器的IP
        String newIp = clientConfig.addServer();
        // 将新服务器的IP添加到ipList中
        redisTemplate.opsForList().rightPush("ipList", newIp);
        // 将新服务器的绑定数量初始化为0
        redisTemplate.opsForHash().put("serverBindings", newIp, 0);
        return newIp;
    }

    /**
     * 检查服务器是否已满
     */
    private boolean isServerFull(String ip) {
        Integer bindingCount = (Integer)redisTemplate.opsForHash().get("serverBindings", ip);
        return bindingCount >= 6;
    }


    /**
     * 寻找下一个未满的服务器
     */
    private String findAvailableServer() {
        List<String> ipList = redisTemplate.opsForList().range("ipList", 0, -1);
        for (String ip : ipList) {
            if (!isServerFull(ip)) {
                return ip;
            }
        }
        return null; // 如果所有服务器都满了，返回null
    }

    /**
     * 更新服务器绑定数量
     */
    private void incrementServerBindingCount(String ip) {
        // 获取当前服务器的绑定数量
        Integer bindingCount = (Integer)redisTemplate.opsForHash().get("serverBindings", ip);
        // 更新绑定数量
        redisTemplate.opsForHash().put("serverBindings", ip, bindingCount + 1);
    }


}
